@@ -10,11 +10,14 @@
 
 #include <string.h>
 #include <stdarg.h>
+
 #ifdef _WIN32
 #ifdef _MSC_VER
 #include <winsock2.h>
 #pragma comment(lib, "ws2_32.lib")
 #define gettimeofday(p1,p2)
+#else /* _MSC_VER */
+#include <sys/time.h>
 #endif /* _MSC_VER */
 #else /* _WIN32 */
 #include <sys/time.h>
@@ -951,6 +954,7 @@ main(int argc, char **argv) {
 	    return(1);
 	}
     }
+    xmlLineNumbersDefault(1);
     if (loaddtd != 0) xmlLoadExtDtdDefaultValue = 6; /* fetch DTDs by default */
     if (noent != 0) xmlSubstituteEntitiesDefault(1);
     if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
